; =======================================================================================
;
; Window message craker macros for MASM
; Created by: Chris Vega
;
; =======================================================================================
;
; Macros for window message (WM) handling. Must be called inside window callback
; function. These macros will generate a set of .if/.elseif/.endif block, using
; a register (eax) to compare for the correct window message, and generate a call 
; to a handler function for the window message. Each handler functions has different
; set of arguments, and all handler functions starts with hWnd parameter to
; specify the owner window of the callback function that made the call to the
; handler function.
;
; Note: The message cracker macros uses constant names for it to find the correct macro
; to generate the proper call to handler function, i.e., using WM_KEYDOWN instead of 100h.
;
; How to use:
; 
; Include both windows.inc and this file to your asm source. And inside window callback 
; function, i.e., WndProc create a .handle_msg_begin/.handle_msg_end block, following 
; this syntax:
;
;		.handle_msg_begin [HWND], [MSG], [WPARAM], [LPARAM], [REG]
;			.handle_msg <window message constant>, <handler function>
;			.handle_msg <window message constant>, <handler function>
;			.handle_msg <window message constant>, <handler function>
;		.handle_msg_end
;			
; 		[HWND], [MSG], [WPARAM], [LPARAM], [REG] are optional, see below.
;
; The code syntax may look similar to this actual codes:
;
;	WndProc proc hWnd:dword, uMsg:dword, wParam:dword, lParam:dword
;		
;		.handle_msg_begin
;			.handle_msg WM_CLOSE, WmClose
;			.handle_msg WM_MOVING, WmMoving
;		.handle_msg_end
;		
;		xor	eax, eax
;		ret
;	WndProc endp
;
; The above example will branch to WmClose whenever WndProc receives WM_CLOSE message,
; or call WmMoving whenever WM_MOVING is received. The macro will assemble it as a
; set of .if/.elseif/.endif statements, as below:
;
; 	mov	eax, uMsg
;	.if eax==WM_CLOSE
;		push hWnd
;		call WmClose
;	.elseif eax==WM_MOVING
;		push lParam
;		push hWnd
;		call WmMoving
;	.endif
;
; Other macros that generate function call does extra extraction/manipulation to 
; parameters before it pushes it to the stack, for the handler function to receive it
; as parameters.
;
; Another important thing to note here is, the message cracking macros uses default naming
; for hWnd, uMsg, lParam and wParam callback parameters, but the defaults may be customized
; by the coder at anytime he/she wanted to. For this reason, the message cracking macros
; can be customized as well, by passing the names of the parameters when you call
; .handle_msg_begin to start the message cracking block, the syntax for this call is:
;
;		.handle_msg_begin [HWND], [MSG], [WPARAM], [LPARAM], [REG]
;
; Where:
;
; [HWND]	The the name for hwnd, default is hWnd
; [MSG] 	The the name for msg, default is uMsg
; [WPARAM]	The the name for wParam, default is wParam
; [LPARAM]	The the name for lParam, default is lParam
; [REG]		The register that the macros will use in generating .if/.elseif/endif statements
;
;
;	WndProc proc hwnd:dword, msg:dword, wparam:dword, lparam:dword
;			
;		.handle_msg_begin hwnd, msg, wparam, lparam
;			.handle_msg WM_CLOSE, WmClose
;			.handle_msg WM_MOVING, WmMoving
;		.handle_msg_end
;			
;		xor	eax, eax
;		ret
;	WndProc endp
;
;
; =======================================================================================
; Adding window message handler macros
; =======================================================================================
;
; The collection of window messages here are those that are found in MSDN, it means, only
; those messages that are documented. All undocumented messages are not included here, for
; the reason, I don't know about them, or because of lack of information on how to implement
; it here.
;
; If you think that the window message you need is not included here, and you want to add
; it to the collection, simply create a new macro at the end of this file with name 
; .handle_<message name>, i.e., for example your message is named WM_EXECUTING, then simply
; add a macro like this (note: the macro must have all 4 parameters, in_hWnd, in_lParam, 
; in_wParam and addr_handler):
;
;	.handle_WM_EXECUTING macro in_hWnd, in_lParam, in_wParam, addr_handler
;		push	in_wParam
;		push	in_lParam
;		push	in_hWnd
;		call	addr_handler
;	endm
;
; your added macro will be called when the callback function received the message. However,
; since this macros only works for constant names, you must declare a constant for your
; message name (if it is not declared in windows.inc file), hence the call will become:
;
;		.handle_msg_begin
;			.handle_msg WM_EXECUTING, WmExecuting
;		.handle_msg_end
;
; =======================================================================================
; Mixing Message Crackers with other codes
; =======================================================================================
;
; The general concept of this macros is to generate .if/.elseif/endif blocks, hence you
; can mix with the messagecracker with your own handlers, something like this:
;
;	WndProc proc hwnd:dword, msg:dword, wparam:dword, lparam:dword
;			
;		.handle_msg_begin hwnd, msg, wparam, lparam
;			.handle_msg WM_CLOSE, WmClose
;
;			.elseif eax==WM_CREATE
;				invoke CreateWindowEx ...
;				invoke CreateWindowEx ...
;				invoke CreateWindowEx ...
;
;			.handle_msg WM_MOVING, WmMoving
;		.handle_msg_end
;			
;		xor	eax, eax
;		ret
;	WndProc endp
;
; In the code above, the part with .elseif will be called normally without generating
; a call to handler function, which means you can handle WM_CREATE without leaving the
; callback function.
;
;
; =======================================================================================
; Notes
; =======================================================================================
;
; Many of the macros that generates calls to handler functions are not yet fully tested by 
; the author. Most of these macros are converted from VC++ message cracker macros that is
; distributed in WindowsX.h header file. I would greatly appreciate it if you can point
; me any macros that are not working properly, and/or generating erroneous call to handler
; function and/or macros that are implemented in the wrong way other than what it supposed to. 
; I would appreciate it also if you can write me feedback on how to improve this include 
; file, either by optimizing codes here or a better way of parameter extractions.
;

option dotname

__wmcrk@reg__ equ <eax>
__wmcrk@hwnd__ equ <hWnd>
__wmcrk@msg__ equ <uMsg>
__wmcrk@lparam__ equ <lParam>
__wmcrk@wparam__ equ <wParam>
__wmcrk@block__ = 0
__wmcrk@count__ = 0

.handle_msg_begin macro hwndname:=<hWnd>, msgname:=<uMsg>, wparamname:=<wParam>, lparamname:=<lParam>, usereg:=<eax>
	if __wmcrk@block__ eq 1
		.err <Message cracker error: .handle_msg does not support nested message checking.>
		exitm
	else
		__wmcrk@block__ = 1
	endif
	
	ifdif <__wmcrk@hwnd__>, <hwndname>
		__wmcrk@hwnd__ textequ <hwndname>
	endif
	
	ifdif <__wmcrk@msg__>, <msgname>
		__wmcrk@msg__ textequ <msgname>
	endif
	
	ifdif <__wmcrk@wparam__>, <wparamname>
		__wmcrk@wparam__ textequ <wparamname>
	endif
	
	ifdif <__wmcrk@lparam__>, <lparamname>
		__wmcrk@lparam__ textequ <lparamname>
	endif
	
	ifdif <__wmcrk@reg__>, <usereg>
		__wmcrk@reg__ textequ <usereg>
	endif
		
	__wmcrk@count__ = 0
	mov __wmcrk@reg__, __wmcrk@msg__
endm

.handle_msg macro message:req, addr_handler:req
	local $statement
	if __wmcrk@block__ eq 0
		.err <Message cracker error: .handle_msg must be called inside .handle_msg_begin/.handle_msg_end block>
		exitm
	endif	
	__wmcrk@count__ = __wmcrk@count__ + 1
	if __wmcrk@count__ eq 1
		$statement catstr <.if >, <__wmcrk@reg__>,<==>,<message>
	else
		$statement catstr <.elseif >,<__wmcrk@reg__>,<==>,<message>
	endif
	%$statement
	.handle_&message& __wmcrk@hwnd__, __wmcrk@lparam__, __wmcrk@wparam__, addr_handler
endm

.handle_msg_end macro
	if __wmcrk@block__ eq 0
		.err <Message cracker error: Bad code block: .handle_msg_end without .handle_msg_begin.>
		exitm
	else
		if __wmcrk@count__ eq 0
			.err <Message cracker error: Unexpected end of block: .handle_msg_begin/.handle_msg_end is empty.>
			exitm
		endif
		
		.endif
		__wmcrk@block__ = 0
		__wmcrk@count__ = 0
	endif
endm

; Helper macro to obtain loword from dword, and convert it back to dword
GetLoWord macro data32
	mov		eax, data32
	and		eax, 0000ffffh
	exitm	<eax>
endm

; Helper macro to obtain hiword from dword, and convert it back to dword
GetHiWord macro data32 
	mov		eax, data32
	shr		eax, 16
	exitm	<eax>
endm

; WM_handler(hWnd:HWND, compactRatio:dword)     [Win16]
.handle_WM_COMPACTING macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpszSectionName:LPSTR)  
.handle_WM_WININICHANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_SYSCOLORCHANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_QUERYNEWPALETTE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndPaletteChange:HWND)
.handle_WM_PALETTEISCHANGING macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndPaletteChange:HWND)
.handle_WM_PALETTECHANGED macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_FONTCHANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, status:DWORD, cJobInQueue:dword)
.handle_WM_SPOOLERSTATUS macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpszDeviceName:LPSTR)
.handle_WM_DEVMODECHANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_TIMECHANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, code:dword)
.handle_WM_POWER macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_QUERYENDSESSION macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fEnding:BOOL)
.handle_WM_ENDSESSION macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, exitCode:dword)
.handle_WM_QUIT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, errCode:dword) [Windows 3.11 only]
.handle_WM_SYSTEMERROR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpCreateStruct:CREATESTRUCT)
.handle_WM_CREATE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpCreateStruct:CREATESTRUCT)
.handle_WM_NCCREATE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_DESTROY macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_NCDESTROY macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fShow:BOOL, status:dword)
.handle_WM_SHOWWINDOW macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fRedraw:BOOL)
.handle_WM_SETREDRAW macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fEnable:BOOL)
.handle_WM_ENABLE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpszText:LPSTR)
.handle_WM_SETTEXT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, cchTextMax:dword, lpszText:LPSTR)
.handle_WM_GETTEXT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_GETTEXTLENGTH macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpwpos:WINDOWPOS)
.handle_WM_WINDOWPOSCHANGING macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpwpos:WINDOWPOS)
.handle_WM_WINDOWPOSCHANGED macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword)
.handle_WM_MOVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, rect:RECT)
.handle_WM_MOVING macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, state:dword, lx:dword, cy:dword)
.handle_WM_SIZE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, edge:dword, rect:RECT)
.handle_WM_SIZING macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_CLOSE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, defCtrl:HWND, initData:dword)
.handle_WM_INITDIALOG macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_QUERYOPEN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpMinMaxInfo:LPMINMAXINFO)
.handle_WM_GETMINMAXINFO macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_PAINT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC)
.handle_WM_ERASEBKGND macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC)
.handle_WM_ICONERASEBKGND macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hrgn:HRGN)
.handle_WM_NCPAINT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fCalcValidRects:BOOL, lpcsp:NCCALCSIZE_PARAMS)
.handle_WM_NCCALCSIZE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword)
.handle_WM_NCHITTEST macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_QUERYDRAGICON macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdrop:HDROP)
.handle_WM_DROPFILES macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, state:dword, hWndActDeact:HWND, fMinimized:dword)
.handle_WM_ACTIVATE macro in_hWnd, in_lParam, in_wParam, addr_handler 
	push	GetHiWord(in_wParam)
	push	in_lParam
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fActivate:BOOL, dwThreadId:dword)
.handle_WM_ACTIVATEAPP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, cmd:dword, uDevice:dword, dwKeys:dword
.handle_WM_APPCOMMAND macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GET_KEYSTATE_LPARAM(in_lParam)
	push	GET_DEVICE_LPARAM(in_lParam)
	push	GET_APPCOMMAND_LPARAM(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fActive:BOOL, hWndActDeact:HWND, fMinimized:BOOL)
.handle_WM_NCACTIVATE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndOldFocus:HWND)
.handle_WM_SETFOCUS macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND: hWndNewFocus:HWND)
.handle_WM_KILLFOCUS macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, vk:dword, fDown:BOOL, cRepeat:dword, flags:dword)
.handle_WM_KEYDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, vk:dword, fDown:BOOL, cRepeat:dword, flags:dword)
.handle_WM_KEYUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, ch:dword, cRepeat:dword)
.handle_WM_CHAR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetLoWord(in_lParam)
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, ch:dword, cRepeat:dword)
.handle_WM_DEADCHAR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetLoWord(in_lParam)
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, vk:dword, fDown:dword, cRepeat:dword, flags:dword)
.handle_WM_SYSKEYDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, vk:dword, fDown:dword, cRepeat:dword, flags:dword)
.handle_WM_SYSKEYUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, ch:dword, cRepeat:dword)
.handle_WM_SYSCHAR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetLoWord(in_lParam)
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, ch:dword, cRepeat:dword)
.handle_WM_SYSDEADCHAR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetLoWord(in_lParam)
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, keyFlags:dword)
.handle_WM_MOUSEMOVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_MOUSELEAVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_LBUTTONDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_LBUTTONDBLCLK macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, keyFlags:dword)
.handle_WM_LBUTTONUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_RBUTTONDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_RBUTTONDBLCLK macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, keyFlags:dword)
.handle_WM_RBUTTONUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_MBUTTONDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_MBUTTONDBLCLK macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, keyFlags:dword)
.handle_WM_XBUTTONUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_XBUTTONDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, keyFlags:dword)
.handle_WM_XBUTTONDBLCLK macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, keyFlags:dword)
.handle_WM_MBUTTONUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, xPos:dword, yPos:dword, zDelta:dword, fwKeys:dword)
.handle_WM_MOUSEWHEEL macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetLoWord(in_wParam)
	push	GetHiWord(in_wParam)
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCMOUSEMOVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_NCMOUSELEAVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCLBUTTONDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCLBUTTONDBLCLK macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCLBUTTONUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCRBUTTONDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCRBUTTONDBLCLK macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCRBUTTONUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCMBUTTONDOWN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	FALSE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fDoubleClick:BOOL, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCMBUTTONDBLCLK macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	TRUE
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, x:dword, y:dword, codeHitTest:dword)
.handle_WM_NCMBUTTONUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndTopLevel:HWND, codeHitTest:dword, UINT msg:dword)
.handle_WM_MOUSEACTIVATE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_CANCELMODE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndCapture:HWND)
.handle_WM_CAPTURECHANGED macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, id:dword)
.handle_WM_TIMER macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hMenu:HMENU)
.handle_WM_INITMENU macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hMenu:HMENU, item:dword, fSystemMenu:BOOL)
.handle_WM_INITMENUPOPUP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hmenu:HMENU, item:dword, hmenuPopup:HMENU, flags:dword)
.handle_WM_MENUSELECT macro in_hWnd, in_lParam, in_wParam, addr_handler
    xor		ebx, ebx
	mov		ebx, wParam
	shr		ebx, 16
	.if		(bx == -1)
	push 	0ffffffffh
	.else
	push 	GetHiWord(in_wParam)
	.endif
	xor		ebx, ebx
	mov		ebx, wParam
	shr		ebx, 16
	.if		(bx or MF_POPUP)
	push	GetLoWord(in_wParam)
	push	in_lParam
	call	GetSubMenu
	push	eax
	.else
	push	0
	.endif
	xor		ebx, ebx
	mov		ebx, wParam
	shr		ebx, 16
	.if		(bx or MF_POPUP)
	push	0
	.else
	push	GetLoWord(in_wParam)
	.endif
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, ch:dword, flags:dword, hmenu:HMENU)
.handle_WM_MENUCHAR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	GetHiWord(in_wParam)
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, id:dword, hWndCtl:HWND, codeNotify:dword)
.handle_WM_COMMAND macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_wParam)
	push	in_lParam
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndCtl:HWND, code:dword, pos:dword)
.handle_WM_HSCROLL macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_wParam)
	push	GetLoWord(in_wParam)
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndCtl:HWND, code:dword, pos:dword)
.handle_WM_VSCROLL macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_wParam)
	push	GetLoWord(in_wParam)
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_CUT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_COPY macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_PASTE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_CLEAR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_UNDO macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fmt:dword)
.handle_WM_RENDERFORMAT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_RENDERALLFORMATS macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_DESTROYCLIPBOARD macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_DRAWCLIPBOARD macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndCBViewer:HWND, lpPaintStruct:PAINTSTRUCT)
.handle_WM_PAINTCLIPBOARD macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	call	GlobalLock
	push	in_lParam	
	push	in_wParam
	push	in_hWnd
	call	addr_handler
	push	in_lParam
	call	GlobalUnlock
endm

; WM_handler(hWnd:HWND, hWndCBViewer:HWND, lprc:RECT)
.handle_WM_SIZECLIPBOARD macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	call	GlobalLock
	push	in_lParam	
	push	in_wParam
	push	in_hWnd
	call	addr_handler
	push	in_lParam
	call	GlobalUnlock
endm

; WM_handler(hWnd:HWND, hWndCBViewer:HWND, code:dword, pos:dword)
.handle_WM_VSCROLLCLIPBOARD macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndCBViewer:HWND, code:dword, pos:dword)
.handle_WM_HSCROLLCLIPBOARD macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, cchMax:dword, rgchName:LPTSTR)
.handle_WM_ASKCBFORMATNAME macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndRemove:HWND, hWndNext:HWND)
.handle_WM_CHANGECBCHAIN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndCursor:HWND, codeHitTest:dword, msg:dword)
.handle_WM_SETCURSOR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, cmd:dword, x:dword, y:dword)
.handle_WM_SYSCOMMAND macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpmcs:MDICREATESTRUCT)
.handle_WM_MDICREATE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndDestroy:HWND)
.handle_WM_MDIDESTROY macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fActive:BOOL, hWndActivate:HWND, in_hWndDeactivate:HWND) 
.handle_WM_MDIACTIVATE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_lParam
	.if in_lParam==in_hWnd
	push	TRUE
	.else
	push	FALSE
	.endif
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndRestore:HWND)
.handle_WM_MDIRESTORE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndCur:HWND, fPrev:BOOL)
.handle_WM_MDINEXT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndMaximize:HWND)
.handle_WM_MDIMAXIMIZE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, helpInfo:HELPINFO)
.handle_WM_HELP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, cmd:dword)
.handle_WM_MDITILE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, cmd:dword)
.handle_WM_MDICASCADE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_MDIICONARRANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_MDIGETACTIVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fRefresh:BOOL, hmenuFrame:HMENU, hmenuWindow:HMENU)
.handle_WM_MDISETMENU macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_CHILDACTIVATE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndSetFocus:HWND, fNext:BOOL)
.handle_WM_NEXTDLGCTL macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, msg:dword, hWndChild:HWND, idChild:dword)
.handle_WM_PARENTNOTIFY macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_wParam)
	push	in_lParam
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, source:dword, in_hWndSource:HWND)
.handle_WM_ENTERIDLE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, fromTrackPopupMenu:BOOL)
.handle_WM_ENTERMENULOOP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, shortcut:BOOL)
.handle_WM_EXITMENULOOP macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_ENTERSIZEMOVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_EXITSIZEMOVE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpmsg:dword)
.handle_WM_GETDLGCODE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC, hWndChild:HWND, type:dword)
.handle_WM_CTLCOLORMSGBOX macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	CTLCOLOR_MSGBOX
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC, hWndChild:HWND, type:dword)
.handle_WM_CTLCOLOREDIT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	CTLCOLOR_EDIT
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC, hWndChild:HWND, type:dword)
.handle_WM_CTLCOLORLISTBOX macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	CTLCOLOR_LISTBOX
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC, hWndChild:HWND, type:dword)
.handle_WM_CTLCOLORBTN macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	CTLCOLOR_BTN
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC, hWndChild:HWND, type:dword)
.handle_WM_CTLCOLORDLG macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	CTLCOLOR_DLG
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC, hWndChild:HWND, type:dword)
.handle_WM_CTLCOLORSCROLLBAR macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	CTLCOLOR_SCROLLBAR
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hdc:HDC, hWndChild:HWND, type:dword)
.handle_WM_CTLCOLORSTATIC macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	CTLCOLOR_STATIC
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(in_hWndCtl:HWND, hfont:HFONT, fRedraw:BOOL)
.handle_WM_SETFONT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_GETFONT macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpDrawItem:DRAWITEMSTRUCT)
.handle_WM_DRAWITEM macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpMeasureItem:MEASUREITEMSTRUCT)
.handle_WM_MEASUREITEM macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpDeleteItem:DELETEITEMSTRUCT)
.handle_WM_DELETEITEM macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lpCompareItem:COMPAREITEMSTRUCT)
.handle_WM_COMPAREITEM macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, vk:dword, hWndListbox:HWND, iCaret:dword)
.handle_WM_VKEYTOITEM macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_wParam)
	push	in_lParam
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, ch:dword, hWndListbox:HWND, iCaret:dword)
.handle_WM_CHARTOITEM macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_wParam)
	push	in_lParam
	push	GetLoWord(in_wParam)
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_QUEUESYNC macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, cid:dword, flags:dword)
.handle_WM_COMMNOTIFY macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, bitsPerPixel:dword, cxScreen:dword, cyScreen:dword)
.handle_WM_DISPLAYCHANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, uEvent:dword, dwEventData:dword)
.handle_WM_DEVICECHANGE macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndContext:HWND, xPos:dword, yPos:dword)
.handle_WM_CONTEXTMENU macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, hWndContext:HWND, copydata:COPYDATASTRUCT)
.handle_WM_COPYDATA macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, idHotKey:dword, fuModifiers, vk)
.handle_WM_HOTKEY macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	GetHiWord(in_lParam)
	push	GetLoWord(in_lParam)
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND)
.handle_WM_CUSTOMHANDLER1 macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lParam:dword)
.handle_WM_CUSTOMHANDLER2 macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, wParam:dword)
.handle_WM_CUSTOMHANDLER3 macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_hWnd
	call	addr_handler
endm

; WM_handler(hWnd:HWND, lParam:dword, wParam:dword)
.handle_WM_CUSTOMHANDLER4 macro in_hWnd, in_lParam, in_wParam, addr_handler
	push	in_wParam
	push	in_lParam
	push	in_hWnd
	call	addr_handler
endm